﻿//
// Copyright (c) 2015, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;
using MindFusion.Geometry.Geometry2D;


namespace MindFusion.Gauges.WinForms.Samples.CS.Compass
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();			

			for (int i = 15; i <=345;)
            {
                Range rng = new Range();
                rng.MinValue = i;
                rng.MaxValue = i + 15;
                rng.Fill = new MindFusion.Drawing.SolidBrush(
                    Color.FromArgb(255, 64, 64, 64));
                rng.Stroke = new MindFusion.Drawing.Pen(
                    Color.FromArgb(255, 32, 32, 32));
                rng.Alignment = Alignment.TrueCenter;

                compass.Scales[1].Ranges.Add(rng);
                i += 30;
            }

            compass.Scales[1].QueryLabelValue += new EventHandler<QueryLabelValueEventArgs>(InnerScale_QueryLabelValue);
        }

        void InnerScale_QueryLabelValue(object sender, QueryLabelValueEventArgs e)
        {
            if (e.CalculatedLabelValue == 0)
                e.NewValue = "N";
            if (e.CalculatedLabelValue == 45)
                e.NewValue = "NE";
            if (e.CalculatedLabelValue == 90)
                e.NewValue = "E";
            if (e.CalculatedLabelValue == 135)
                e.NewValue = "SE";
            if (e.CalculatedLabelValue == 180)
                e.NewValue = "S";
            if (e.CalculatedLabelValue == 225)
                e.NewValue = "SW";
            if (e.CalculatedLabelValue == 270)
                e.NewValue = "W";
            if (e.CalculatedLabelValue == 315)
                e.NewValue = "NW";
        }

        private void compass_PrepaintBackground(object sender, PrepaintEventArgs e)
        {
            //Cancels the painting of the default background
            e.CancelDefaultPainting = true;

            RectangleF boundingRect = new RectangleF(0, 0, e.Element.RenderSize.Width, e.Element.RenderSize.Height);

            //Background layer 1
            LinearGradientBrush fill1 = new LinearGradientBrush(
                new Point(0,0),
                new PointF(boundingRect.Right, boundingRect.Bottom),//1, 1
                Color.FromArgb(255, 148, 148, 148),
                Color.FromArgb(255, 48, 48, 48));

            Blend blend = new Blend();
            blend.Positions = new float[] {0, 0.2f, 0.8f, 1 };
            blend.Factors = new float[] {0, 0, 1, 1 };

            fill1.Blend = blend;
            e.Graphics.FillEllipse(fill1, boundingRect);

            fill1.Dispose();

            boundingRect.Inflate(-0.013f * boundingRect.Width, -0.013f * boundingRect.Height);

            //Background layer 2
            LinearGradientBrush fill2 = new LinearGradientBrush(
                new PointF(boundingRect.Right, boundingRect.Bottom),
                new Point(0, 0),
                Color.FromArgb(255, 148, 148, 148),
                Color.FromArgb(255, 48, 48, 48));

            fill2.Blend = blend;
            e.Graphics.FillEllipse(fill2, boundingRect);

            fill2.Dispose();

            boundingRect.Inflate(-0.013f * boundingRect.Width, -0.013f * boundingRect.Height);

            //Background layer 3
            Brush fill3 = new SolidBrush(
                Color.FromArgb(255, 222, 222, 222));
            e.Graphics.FillEllipse(fill3, boundingRect);

            fill3.Dispose();

        }

        private void compass_PrepaintForeground(object sender, PrepaintEventArgs e)
        {
            //Cancels the painting of the default foreground
            e.CancelDefaultPainting = true;

            RectangleF boundingRect = new RectangleF(0, 0, e.Element.RenderSize.Width, e.Element.RenderSize.Height);

            GraphicsPath areaPath = new GraphicsPath();
            areaPath.AddEllipse(boundingRect);

            MindFusion.Drawing.PathGradientBrush area1Fill = 
                new MindFusion.Drawing.PathGradientBrush(
                Color.FromArgb(96, Color.White), Color.Transparent);
            area1Fill.Path = areaPath;

            MindFusion.Drawing.PathGradientBrush area2Fill = 
                new MindFusion.Drawing.PathGradientBrush(
                Color.FromArgb(64, Color.White), Color.Transparent);
            area2Fill.Path = areaPath;

            ArcArea area1 = new ArcArea();
            area1.StartAngle = 160;
            area1.EndAngle = -20;
            area1.Margin = new Thickness(0.03f);
            area1.Fill = area1Fill;

            ArcArea area2 = new ArcArea();
            area2.StartAngle = 140;
            area2.EndAngle = -40;
            area2.Margin = new Thickness(0.03f);
            area2.Fill = area2Fill;

            area1.RelativeCoordinates = area2.RelativeCoordinates = false;

            e.PaintVisualElement(area1, e.Element.RenderSize);
            e.PaintVisualElement(area2, e.Element.RenderSize);

            areaPath.Dispose();
            
        }

        private void compass_PrepaintScale(object sender, PrepaintEventArgs e)
        {
            if (e.Element.Name == "InnerScale")
            {
                GraphicsState state = e.Graphics.Save();
                e.Graphics.ScaleTransform(e.Element.RenderSize.Width, e.Element.RenderSize.Height);

                Pen commonStroke = new Pen(
                    Color.FromArgb(255, 32, 32, 32), 0.001f);
                Brush lightFill = new SolidBrush(
                    Color.FromArgb(255, 191, 191, 191));
                Brush darkFill = new SolidBrush(
                    Color.FromArgb(255, 128, 128, 128));

                // Four main ploygons of the "sun"

                //Polygon 1
                GraphicsPath path1 = new GraphicsPath();

                PointF pt1 = new PointF(0.44f, 0.5f);
                PointF pt2 = new PointF(0.5f, 0.15f);
                PointF pt3 = new PointF(0.5f, 0.85f);
                PointF pt4 = new PointF(0.56f, 0.5f);

                path1.AddPolygon(new PointF[] { pt1, pt2, pt3, pt4 });

                e.Graphics.FillPolygon(lightFill,
                    new PointF[] { pt1, pt2, pt3, pt4 });

                Matrix rotateMatrix = new Matrix();
                rotateMatrix.RotateAt(90, new PointF(0.5f, 0.5f));

                //Polygon 2
                path1.Transform(rotateMatrix);
                e.Graphics.FillPath(lightFill, path1);

                path1.Dispose();

                //Polygon 3
                GraphicsPath path2 = new GraphicsPath();

                path2.AddPolygon(new PointF[] { pt4, pt2, pt3, pt1 });
                e.Graphics.FillPolygon(darkFill,
                    new PointF[] { pt4, pt2, pt3, pt1 });

                //Polygon 4
                path2.Transform(rotateMatrix);
                e.Graphics.FillPath(darkFill, path2);

                path2.Dispose();

                PointF pt5 = new PointF(0.5f, 0.56f);
                PointF pt6 = new PointF(0.15f, 0.5f);
                PointF pt7 = new PointF(0.5f, 0.44f);
                PointF pt8 = new PointF(0.85f, 0.5f);

                e.Graphics.DrawPolygon(commonStroke,
                    new PointF[] { pt1, pt2, pt4, pt3 });
                e.Graphics.DrawPolygon(commonStroke,
                    new PointF[] { pt5, pt6, pt7, pt8 });


                //Four minor rays of the "sun"
                GraphicsPath path3 = new GraphicsPath();

                PointF p1 = new PointF(0.46f, 0.5f);
                PointF p2 = new PointF(0.3f, 0.3f);
                PointF p3 = new PointF(0.5f, 0.46f);
                PointF p4 = new PointF(0.7f, 0.3f);
                PointF p5 = new PointF(0.53f, 0.5f);
                PointF p6 = new PointF(0.7f, 0.7f);
                PointF p7 = new PointF(0.5f, 0.53f);
                PointF p8 = new PointF(0.3f, 0.7f);

                path3.AddPolygon(new PointF[] { 
                    p1, p2, p3, p4, p5, p6, p7, p8 });
                e.Graphics.FillPolygon(lightFill,
                    new PointF[] { p1, p2, p3, p4, p5, p6, p7, p8 });

                e.Graphics.DrawPolygon(commonStroke,
                    new PointF[] { p1, p2, p3, p4, p5, p6, p7, p8 });

                path3.Dispose();

                //Center of the "sun"
                RectangleF boundingRect = new RectangleF(0, 0, 1, 1);

                boundingRect.Inflate(-0.37f, -0.37f);

                e.Graphics.DrawEllipse(commonStroke, boundingRect);
                e.Graphics.FillEllipse(lightFill, boundingRect);

                boundingRect.Inflate(-0.01f, -0.01f);

                e.Graphics.DrawEllipse(commonStroke, boundingRect);
                e.Graphics.FillEllipse(
                    new SolidBrush(Color.FromArgb(255, 222, 222, 222))
                    , boundingRect
                    );

                commonStroke.Dispose();
                lightFill.Dispose();
                darkFill.Dispose();
                rotateMatrix.Dispose();
                e.Graphics.Restore(state);
            }
        }

        private void compass_PaintPointer(object sender, CustomPaintEventArgs e)
        {
            if (e.Element.Name == "MainPointer")
            {
                GraphicsState state = e.Graphics.Save();
                e.Graphics.ScaleTransform(e.Element.RenderSize.Width, e.Element.RenderSize.Height);

                RectangleF boundingRect = new RectangleF(0, 0, 1, 1);

                Pen commonStroke = new Pen(Color.FromArgb(255, 32, 32, 32), 0.015f);

                GraphicsPath fill1path = new GraphicsPath();

                PointF pt1 = new PointF(0.5f, 0);
                PointF pt2 = new PointF(1, 0.5f);
                PointF pt3 = new PointF(0.5f, 1);

                fill1path.AddPolygon(new PointF[] { pt1, pt2, pt3 });

                PathGradientBrush fill1 = new PathGradientBrush(fill1path);

                fill1.CenterPoint = new PointF(0.5f, 0);

                ColorBlend fill1blend = new ColorBlend();
                fill1blend.Positions = new float[] { 0, 0.3f, 1 };
                fill1blend.Colors = new Color[]{
                    Color.FromArgb(255, 193, 0, 0),
                    Color.FromArgb(255, 255, 0, 0),
                    Color.FromArgb(255, 102, 0, 0)
                };

                fill1.InterpolationColors = fill1blend;

                e.Graphics.FillPolygon(fill1, new PointF[] { pt1, pt2, pt3 });

                PointF pt4 = new PointF(0, 0.5f);

                fill1path.AddPolygon(new PointF[] { pt1, pt4, pt3 });

                PathGradientBrush fill2 = new PathGradientBrush(fill1path);

                fill2.CenterPoint = new PointF(0.5f, 0);
                ColorBlend fill2blend = new ColorBlend();
                fill2blend.Positions = new float[] { 0, 0.3f, 1 };
                fill2blend.Colors = new Color[]{
                    Color.FromArgb(255, 193, 193, 193),
                    Color.FromArgb(255, 204, 204, 204),
                    Color.FromArgb(255, 102, 102, 102)
                };

                fill2.InterpolationColors = fill2blend;

                e.Graphics.FillPolygon(fill2, new PointF[] { pt1, pt4, pt3 });

                e.Graphics.DrawPolygon(commonStroke, new PointF[] { pt1, pt2, pt3, pt4 });

                PointF center = new PointF(boundingRect.Width / 2, boundingRect.Height / 2);
                SizeF size = new SizeF(0.03f, 0.3f);

                e.Graphics.DrawEllipse(commonStroke, new RectangleF(new PointF(center.X - size.Width / 2f, center.Y - size.Height / 2f), size));
                e.Graphics.FillEllipse(new SolidBrush(Color.DimGray), new RectangleF(new PointF(center.X - size.Width / 2f, center.Y - size.Height / 2f), size));

                fill1.Dispose();
                fill2.Dispose();
                fill1path.Dispose();
                e.Graphics.Restore(state);
            }
        }
    }
}